"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NotionService = void 0;
const fetchData_1 = require("../../util/fetchData");
/**
 * 간단한 병렬 제한 큐
 */
class ParallelLimiter {
    running = 0;
    queue = [];
    limit;
    constructor(limit) {
        this.limit = limit;
    }
    async add(fn) {
        return new Promise((resolve, reject) => {
            const run = () => {
                this.running++;
                fn()
                    .then(resolve)
                    .catch(reject)
                    .finally(() => {
                    this.running--;
                    this.next();
                });
            };
            if (this.running < this.limit) {
                run();
            }
            else {
                this.queue.push(run);
            }
        });
    }
    next() {
        if (this.queue.length > 0 && this.running < this.limit) {
            const run = this.queue.shift();
            if (run)
                run();
        }
    }
    setLimit(limit) {
        this.limit = limit;
        // 새 제한 설정 후 큐에서 가능한 작업 실행
        while (this.running < this.limit && this.queue.length > 0) {
            this.next();
        }
    }
}
class NotionService {
    reporter;
    parallelLimit;
    enableCaching;
    cache;
    limiter;
    constructor(options) {
        this.reporter = options.reporter;
        this.parallelLimit = options.parallelLimit || 5; // 기본 동시 요청 수
        this.enableCaching = options.enableCaching !== false; // 기본값은 캐싱 활성화
        this.cache = new Map();
        this.limiter = new ParallelLimiter(this.parallelLimit);
    }
    /**
     * 데이터베이스 쿼리
     */
    async queryDatabase(databaseId, body = {}) {
        const cacheKey = `database-${databaseId}-${JSON.stringify(body)}`;
        if (this.enableCaching && this.cache.has(cacheKey)) {
            this.reporter.info(`[CACHE] Using cached database query for ${databaseId}`);
            return this.cache.get(cacheKey);
        }
        const databaseUrl = `databases/${databaseId}/query`;
        try {
            const result = await (0, fetchData_1.fetchPostWithRetry)(databaseUrl, body);
            if (this.enableCaching) {
                this.cache.set(cacheKey, result);
            }
            this.reporter.info(`[SUCCESS] Database query ${databaseId} - results: ${result?.results?.length || 0}`);
            return result;
        }
        catch (error) {
            this.reporter.error(`[ERROR] Failed to query database ${databaseId}: ${error}`);
            throw error;
        }
    }
    /**
     * 페이지의 자식 블록 가져오기
     */
    async getPageBlocks(pageId) {
        const cacheKey = `page-blocks-${pageId}`;
        if (this.enableCaching && this.cache.has(cacheKey)) {
            this.reporter.info(`[CACHE] Using cached page blocks for ${pageId}`);
            return this.cache.get(cacheKey);
        }
        const pageUrl = `blocks/${pageId}/children?page_size=100`;
        try {
            const data = await (0, fetchData_1.fetchGetWithRetry)(pageUrl);
            const blocks = data.results;
            if (this.enableCaching) {
                this.cache.set(cacheKey, blocks);
            }
            return blocks;
        }
        catch (error) {
            this.reporter.error(`[ERROR] Failed to get page blocks ${pageId}: ${error}`);
            throw error;
        }
    }
    /**
     * 여러 페이지의 블록 병렬 처리
     */
    async getMultiplePagesBlocks(pageIds) {
        this.reporter.info(`[NOTION] Fetching blocks for ${pageIds.length} pages in parallel (limit: ${this.parallelLimit})`);
        const tasks = pageIds.map((pageId) => this.limiter.add(async () => {
            try {
                const blocks = await this.getPageBlocks(pageId);
                return { pageId, blocks };
            }
            catch (error) {
                this.reporter.warn(`[WARNING] Failed to fetch blocks for page ${pageId}: ${error}`);
                return { pageId, blocks: [] };
            }
        }));
        const results = await Promise.all(tasks);
        return results.reduce((acc, { pageId, blocks }) => {
            acc[pageId] = blocks;
            return acc;
        }, {});
    }
    /**
     * 캐시 초기화
     */
    clearCache() {
        this.reporter.info(`[NOTION] Clearing service cache - entries: ${this.cache.size}`);
        this.cache.clear();
    }
    /**
     * 병렬 처리 제한 설정
     */
    setParallelLimit(limit) {
        this.reporter.info(`[NOTION] Updated parallel request limit to ${limit}`);
        this.parallelLimit = limit;
        this.limiter.setLimit(limit);
    }
}
exports.NotionService = NotionService;
